@@ -18,121 +18,8 @@ import shutil
 import pathlib
 from FreeTAKServer.controllers.configuration.MainConfig import MainConfig
 
-try:
-    import requests
-except ImportError:
-    subprocess.run(["pip3", "install", "requests"], capture_output=True)
-import hashlib
-
-
-def _utc_time_from_datetime(date):
-    fmt = '%y%m%d%H%M'
-    if date.second > 0:
-        fmt += '%S'
-    if date.tzinfo is None:
-        fmt += 'Z'
-    else:
-        fmt += '%z'
-    return date.strftime(fmt)
-
-
-def revoke_certificate(username, revoked_file=None, ca_pem = MainConfig.CA, ca_key = MainConfig.CAkey, crl_file = MainConfig.CRLFile, user_cert_dir=MainConfig.certsPath, crl_path=MainConfig.CRLFile):
-    """
-    Function to create/update a CRL with revoked user certificates
-    :param ca_pem: The path to your CA PEM file
-    :param ca_key: The Path to your CA key file
-    :param revoked_file: Path to JSON file to be used as a DB for revocation
-    :param crl_file: Path to CRL file
-    :param user_cert_dir: Path to director containing all issued user PEM files
-    :param username: the username to Revoke
-    :param crl_path: The path to your previous CRL file to be loaded and updated
-    :return: bool
-    """
-
-    import os
-    import json
-    from OpenSSL import crypto
-    from datetime import datetime
-    data = {}
-    certificate = crypto.load_certificate(crypto.FILETYPE_PEM, open(ca_pem, mode="rb").read())
-    private_key = crypto.load_privatekey(crypto.FILETYPE_PEM, open(ca_key, mode="r").read())
-    if crl_path and os.path.exists(crl_path):
-        crl = crypto.load_crl(crypto.FILETYPE_PEM, open(crl_path, mode="rb").read())
-    else:
-        crl = crypto.CRL()
-        if revoked_file and os.path.exists(revoked_file):
-            with open(revoked_file, 'r') as json_file:
-                data = json.load(json_file)
-
-    for cert in os.listdir(user_cert_dir):
-        if cert.lower() == f"{username.lower()}.pem":
-            with open(MainConfig.certsPath+'/'+cert, 'rb') as cert:
-                revoked_cert = crypto.load_certificate(crypto.FILETYPE_PEM, cert.read())
-            data[str(revoked_cert.get_serial_number())] = username
-            break
-
-    for key in data:
-        revoked_time = _utc_time_from_datetime(datetime.utcnow())
-        revoked = crypto.Revoked()
-        revoked.set_serial(format(int(key), "02x").encode())
-        revoked.set_rev_date(bytes(revoked_time, encoding='utf8'))
-        crl.add_revoked(revoked)
-    crl.sign(certificate, private_key, b"sha256")
-    if revoked_file:
-        with open(revoked_file, 'w+') as json_file:
-            json.dump(data, json_file)
-
-    with open(crl_file, 'wb') as f:
-        f.write(crl.export(cert=certificate, key=private_key, digest=b"sha256"))
-
-    delete = 0
-    with open(ca_pem, "r") as f:
-        lines = f.readlines()
-    with open(ca_pem, "w") as f:
-        for line in lines:
-            if delete:
-                continue
-            elif line.strip("\n") != "-----BEGIN X509 CRL-----":
-                f.write(line)
-            else:
-                delete = 1
-
-    with open(ca_pem, "ab") as f:
-        f.write(crl.export(cert=certificate, key=private_key, digest=b"sha256"))
-
-
-def send_data_package(server: str, dp_name: str = "user.zip") -> bool:
-    """
-    Function to send data package to server
-    :param server: Server address where the package will be uploaded
-    :param dp_name: Name of the zip file to upload
-    :return: bool
-    """
-    file_hash = hashlib.sha256()
-    block_size = 65536
-    with open(dp_name, 'rb') as f:
-        fb = f.read(block_size)
-        while len(fb) > 0:
-            file_hash.update(fb)
-            fb = f.read(block_size)
-
-    with open(dp_name, 'rb') as f:
-        s = requests.Session()
-        r = s.post(f'http://{server}:8080/Marti/sync/missionupload?hash={file_hash.hexdigest()}'
-                   f'&filename={dp_name}'
-                   f'&creatorUid=atakofthecerts',
-                   files={"assetfile": f.read()},
-                   headers={'Expect': '100-continue'})
-        if r.status_code == 200:
-            p_r = s.put(f'http://{server}:8080/Marti/api/sync/metadata/{file_hash.hexdigest()}/tool')
-            return True
-        else:
-            print("Something went wrong uploading DataPackage!")
-            return False
-
-
-def generate_zip(server_address: str = None, server_filename: str = "pubserver.p12", user_filename: str = "user.p12",
-                 cert_password: str = MainConfig.password, ssl_port: str = "8089") -> None:
+def generate_zip(server_address: str = None, server_filename: str = "pubserver.p12",
+                     user_filename: str = "user.p12", cert_password: str = MainConfig.password) -> None:
     """
     A Function to generate a Client connection Data Package (DP) from a server and user p12 file in the current
     working directory.
@@ -140,7 +27,7 @@ def generate_zip(server_address: str = None, server_filename: str = "pubserver.p
     :param server_filename: The filename of the server p12 file default is pubserver.p12
     :param user_filename: The filename of the server p12 file default is user.p12
     :param cert_password: The password for the certificate files
-    :param ssl_port: The port used for SSL CoT, defaults to 8089
+
     """
     pref_file_template = Template("""<?xml version='1.0' standalone='yes'?>
     <preferences>
@@ -210,8 +97,9 @@ def generate_zip(server_address: str = None, server_filename: str = "pubserver.p
         pref_file.write(pref)
     with open('./MANIFEST/manifest.xml', 'w') as manifest_file:
         manifest_file.write(man)
+    print("Generating Data Package: " + username + ".zip")
     copyfile(MainConfig.p12Dir, "./" + folder + "/" + server_filename)
-    copyfile(MainConfig.certsPath + "/" + user_filename, "./" + folder + "/" + user_filename)
+    copyfile("./" + user_filename, "./" + folder + "/" + user_filename)
     zipf = zipfile.ZipFile(f"{username}.zip", 'w', zipfile.ZIP_DEFLATED)
     for root, dirs, files in os.walk('./' + folder):
         for file in files:
@@ -229,6 +117,7 @@ def generate_zip(server_address: str = None, server_filename: str = "pubserver.p
         os.makedirs("./MANIFEST")
     with open('./MANIFEST/manifest.xml', 'w') as manifest_parent:
         manifest_parent.write(man_parent)
+    print(f"Generating Main Data Package: {username}_DP.zip")
     copyfile(f"./{username}.zip", f"./{parentfolder}/{username}.zip")
     zipp = zipfile.ZipFile(str(pathlib.PurePath(pathlib.Path(MainConfig.clientPackages), pathlib.Path(f"{username}.zip"))), 'w', zipfile.ZIP_DEFLATED)
     for root, dirs, files in os.walk('./' + parentfolder):
@@ -260,16 +149,12 @@ class AtakOfTheCerts:
     def __exit__(self, exc_type, exc_val, exc_tb):
         return None
 
-    def generate_ca(self, expiry_time_secs: int = 31536000) -> None:
+    def generate_ca(self) -> None:
         """
         Generate a CA certificate
         """
         if not os.path.exists(self.cakeypath):
             print("Cannot find CA locally so generating one")
-            if not os.path.exists(os.path.dirname(self.cakeypath)):
-                print("the directory for storing certificates doesn't exist.")
-                print("Creating one at " + os.path.dirname(self.cakeypath))
-                os.makedirs(os.path.dirname(self.cakeypath))
             ca_key = crypto.PKey()
             ca_key.generate_key(crypto.TYPE_RSA, 2048)
             cert = crypto.X509()
@@ -277,7 +162,7 @@ class AtakOfTheCerts:
             cert.set_serial_number(0)
             cert.set_version(2)
             cert.gmtime_adj_notBefore(0)
-            cert.gmtime_adj_notAfter(expiry_time_secs)
+            cert.gmtime_adj_notAfter(31536000)
             cert.set_issuer(cert.get_subject())
             cert.add_extensions([crypto.X509Extension(b'basicConstraints', False, b'CA:TRUE'),
                                  crypto.X509Extension(b'keyUsage', False, b'keyCertSign, cRLSign')])
@@ -287,33 +172,12 @@ class AtakOfTheCerts:
             f = open(self.cakeypath, "wb")
             f.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, ca_key))
             f.close()
+            print("CA key Stored Here: " + self.cakeypath)
 
             f = open(self.capempath, "wb")
             f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
             f.close()
-
-            # append empty crl
-            crl = crypto.CRL()
-            crl.sign(cert, ca_key, b"sha256")
-
-            with open(MainConfig.CRLFile, 'wb') as f:
-                f.write(crl.export(cert=cert, key=ca_key, digest=b"sha256"))
-
-            delete = 0
-            with open(self.capempath, "r") as f:
-                lines = f.readlines()
-            with open(self.capempath, "w") as f:
-                for line in lines:
-                    if delete:
-                        continue
-                    elif line.strip("\n") != "-----BEGIN X509 CRL-----":
-                        f.write(line)
-                    else:
-                        delete = 1
-
-            with open(self.capempath, "ab") as f:
-                f.write(crl.export(cert=cert, key=ca_key, digest=b"sha256"))
-
+            print("CA pem Stored Here: " + self.capempath)
         else:
             print("CA found locally, not generating a new one")
 
@@ -324,67 +188,81 @@ class AtakOfTheCerts:
         """
         if os.path.exists(keypath):
             print("Certificate file exists, aborting.")
+            print(keypath)
         else:
             print("Generating Key...")
             self.key.generate_key(crypto.TYPE_RSA, 2048)
             f = open(keypath, "wb")
             f.write(crypto.dump_privatekey(crypto.FILETYPE_PEM, self.key))
             f.close()
+            print("Key Stored Here: " + keypath)
 
-    def _generate_certificate(self, common_name: str, p12path: str, pempath: str = MainConfig.pemDir,
-                              expiry_time_secs: int = 31536000) -> None:
+    def _generate_certificate(self, cn: str = "Server", pempath: str = MainConfig.pemDir, p12path: str = MainConfig.p12Dir) -> None:
         """
         Create a certificate and p12 file
         :param cn: Common Name for certificate
         :param pempath: String filepath for the pem file created
         :param p12path: String filepath for the p12 file created
-        :param expiry_time_secs: length of time in seconds that the certificate is valid for, defaults to 1 year
         """
-        if not os.path.exists(pempath):
-            ca_key = crypto.load_privatekey(crypto.FILETYPE_PEM, open(self.cakeypath).read())
-            ca_pem = crypto.load_certificate(crypto.FILETYPE_PEM, open(self.capempath, 'rb').read())
-            serial_number = random.getrandbits(64)
-            chain = (ca_pem,)
-            cert = crypto.X509()
-            cert.get_subject().CN = common_name
-            cert.set_serial_number(serial_number)
-            cert.gmtime_adj_notBefore(0)
-            cert.gmtime_adj_notAfter(expiry_time_secs)
-            cert.set_issuer(ca_pem.get_subject())
-            cert.set_pubkey(self.key)
-            cert.set_version(2)
-            cert.sign(ca_key, "sha256")
-            p12 = crypto.PKCS12()
-            p12.set_privatekey(self.key)
-            p12.set_certificate(cert)
-            p12.set_ca_certificates(tuple(chain))
-            p12data = p12.export(passphrase=bytes(self.CERTPWD, encoding='UTF-8'))
-            with open(p12path, 'wb') as p12file:
-                p12file.write(p12data)
+        if os.path.exists(pempath):
+            print("Certificate File Exists, aborting.")
+            print(pempath)
+            return None
+        cakey = crypto.load_privatekey(crypto.FILETYPE_PEM, open(self.cakeypath).read())
+        capem = crypto.load_certificate(crypto.FILETYPE_PEM, open(self.capempath, 'rb').read())
+        serialnumber = random.getrandbits(64)
+        chain = (capem,)
+        cert = crypto.X509()
+        cert.get_subject().CN = cn
+        cert.set_serial_number(serialnumber)
+        cert.gmtime_adj_notBefore(0)
+        cert.gmtime_adj_notAfter(315360000)
+        cert.set_issuer(capem.get_subject())
+        cert.set_pubkey(self.key)
+        cert.set_version(2)
+        cert.sign(cakey, "sha256")
+        print('cert genned')
+        p12 = crypto.PKCS12()
+        p12.set_privatekey(self.key)
+        p12.set_certificate(cert)
+        p12.set_ca_certificates(tuple(chain))
+        p12data = p12.export(passphrase=bytes(self.CERTPWD, encoding='UTF-8'))
+        with open(p12path, 'wb') as p12file:
+            p12file.write(p12data)
+            print("P12 Stored Here: " + p12path)
 
-            if os.path.exists(pempath):
-                print("Certificate File Exists, aborting.")
-            else:
-                f = open(pempath, "wb")
-                f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
-                f.close()
+        if os.path.exists(pempath):
+            print("Certificate File Exists, aborting.")
+            print(pempath)
         else:
-            pass
-    def bake(self, common_name: str, cert: str = "user", expiry_time_secs: int = 31536000) -> None:
+            f = open(pempath, "wb")
+            f.write(crypto.dump_certificate(crypto.FILETYPE_PEM, cert))
+            f.close()
+            print("PEM Stored Here: " + pempath)
+
+    def bake(self, cn: str, cert: str = "user") -> None:
         """
         Wrapper for creating certificate and all files needed
-        :param common_name: Common Name of the the certificate
+        :param cn: Common Name of the the certificate
         :param cert: Type of cert being created "user" or "server"
-        :param expiry_time_secs: length of time in seconds that the certificate is valid for, defaults to 1 year
         """
-        keypath = MainConfig.certsPath+f"/{common_name}.key"
-        pempath = MainConfig.certsPath+f"/{common_name}.pem"
-        p12path = MainConfig.certsPath+f"/{common_name}.p12"
+        import os
+        keypath = f"./{cn}.key"
+        pempath = f"./{cn}.pem"
+        p12path = f"./{cn}.p12"
         self._generate_key(keypath)
-        self._generate_certificate(common_name=common_name, pempath=pempath, p12path=p12path, expiry_time_secs=expiry_time_secs)
+        self._generate_certificate(cn, pempath, p12path)
+        os.remove(keypath)
         if cert.lower() == "server":
             copyfile(keypath, keypath + ".unencrypted")
 
+    def bake_startup(self, cn: str = 'Server') -> None:
+        self.generate_ca()
+        self._generate_key(MainConfig.keyDir)
+        print('key generated')
+        self._generate_certificate(cn, MainConfig.pemDir, MainConfig.p12Dir)
+        copyfile(MainConfig.keyDir, MainConfig.unencryptedKey)
+
     @staticmethod
     def copy_server_certs(server_name: str = "pubserver") -> None:
         """
@@ -402,19 +280,116 @@ class AtakOfTheCerts:
             return None
         if not os.path.exists(dest + "/Certs"):
             os.makedirs(dest + "/Certs")"""
+        print("Copying ./" + server_name + ".key to :" + MainConfig.keyDir)
         copyfile("./" + server_name + ".key", MainConfig.keyDir)
+        print("Done")
+        print("Copying ./" + server_name + ".key to :" + MainConfig.unencryptedKey)
         copyfile("./" + server_name + ".key", MainConfig.unencryptedKey)
+        print("Done")
+        print("Copying ./" + server_name + ".pem to :" + MainConfig.pemDir)
         copyfile("./" + server_name + ".pem", MainConfig.pemDir)
+        print("Done")
+        """print("Copying ./ca.pem to :" + MainConfig.CA)
+        copyfile("./ca.pem", MainConfig.CA)"""
+        print("Done")
 
-    def generate_auto_certs(self, ip: str, copy: bool = False, expiry_time_secs: int = 31536000) -> None:
+    def generate_auto_certs(self, ip: str, copy: bool = False) -> None:
         """
         Generate the basic files needed for a new install of FTS
         :param ip: A string based ip address or FQDN that clients will use to connect to the server
         :param copy: Whether to copy server files to FTS expected locations
-        :param expiry_time_secs: length of time in seconds that the certificate is valid for, defaults to 1 year
         """
-        self.bake("pubserver", "server", expiry_time_secs)
-        self.bake("user", "user", expiry_time_secs)
+        self.bake("pubserver", "server")
+        self.bake("user", "user")
         if copy is True:
             self.copy_server_certs()
         generate_zip(server_address=ip)
+
+if __name__ == '__main__':
+    """if True:
+        AtakOfTheCerts().bake("testcert")
+        generate_zip(user_filename='testcert.p12')"""
+    VERSION = "0.3.5"
+    help_txt = "This Python script is to be used to generate the certificate files needed for \n" \
+               "FTS Version 1.3 and above to allow for SSL/TLS connections between Server and \n" \
+               "Client.\n\n" \
+               "This script works in the current working directory (the folder you are \n" \
+               "currently in)\n\n" \
+               "The .p12 files generated will need to be copied to ATAK clients\n" \
+               "the default password set on the .p12 files is atakatak\n" \
+               "The Server .key and .pem file will ne needed on the FTS server as per the MainConfig.py\n" \
+               "The ca.pem is also needed for the MainConfig.py\n" \
+               "the default password set on the .p12 files is atakatak, this can be overridden\n\n" \
+               "Arguments:\n" \
+               "-h --help : to open help\n" \
+               "-v --version : to print the version number of the script\n" \
+               "-p --password : to change the password for the p12 files from the default atakatak\n" \
+               "-a --automated : to run the script in a headless mode to auto generate ca,server and user certs " \
+               "for a fresh install\n" \
+               "-c --copy : Use this in conjunction with -a to copy the server certs needed into the default location for FTS\n" \
+               "-i --ip : The IP address of the server that clients will be accessing it on\n\n"
+    AUTO = False
+    COPY = False
+    IP = False
+    CERTPWD = "atakatak"
+    cmd_args = sys.argv
+    arg_list = cmd_args[1:]
+    stort_opts = "avhci:p:"
+    long_opts = ["automated", "version", "help", "copy", "ip", "password"]
+    args, values = getopt.getopt(arg_list, stort_opts, long_opts)
+    for current_arg, current_val in args:
+        if current_arg in ("-h", "--help"):
+            print(help_txt)
+            exit(1)
+        if current_arg in ("-v", "--version"):
+            print(VERSION)
+            exit(1)
+        if current_arg in ("-p", "--password"):
+            CERTPWD = current_val
+        if current_arg in ("-a", "--automated"):
+            AUTO = True
+        if current_arg in ("-c", "--copy"):
+            COPY = True
+        if current_arg in ("-i", "--ip"):
+            IP = current_val
+
+    with AtakOfTheCerts() as aotc:
+        aotc.generate_ca()
+    if AUTO:
+        if IP is False:
+            IP = str(input("Enter IP address or FQDN that clients will use to connect to FTS: "))
+        with AtakOfTheCerts() as aotc:
+            aotc.generate_auto_certs(copy=COPY, ip=IP)
+    else:
+        server_p12 = None
+        users_p12 = []
+        server_question = input("Would you like to generate a server certificate? y/n ")
+        if server_question.lower() == "y":
+            with AtakOfTheCerts(CERTPWD) as aotc:
+                IP = str(input("Enter IP address or FQDN that clients will use to connect to FTS: "))
+                aotc.bake(cn=IP, cert="server")
+                server_p12 = "./" + IP + ".p12"
+            copy_question = input("Would you like to copy the server certificate files where needed for FTS? y/n ")
+            if server_question.lower() == "y":
+                aotc.copy_server_certs(server_name=IP)
+        user_question = input("Would you like to generate a user certificate? y/n ")
+        if user_question.lower() == "y":
+            while True:
+                with AtakOfTheCerts(CERTPWD) as aotc:
+                    cn = input("Username: ")
+                    if len(cn) == 0:
+                        break
+                    aotc.bake(cn, cert="user")
+                    users_p12.append("./" + cn + ".p12")
+                    cont = input("Generate another? y/n ")
+                    if cont.lower() != "y":
+                        break
+            generate_zip_question = input(
+                "Would you like to generate Data Packages for each user just created? y/n ")
+            if generate_zip_question.lower() == "y":
+                while server_p12 is None:
+                    server_p12 = input("Enter path to server p12 file e.g ./pubserver.p12 : ")
+                while IP is False:
+                    IP = str(input("Enter IP address or FQDN that clients will use to connect to FTS: "))
+                for user in users_p12:
+                    generate_zip(server_address=IP, server_filename=server_p12, user_filename=user)
\ No newline at end of file
